import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import com.wingarc.svf.pdfarchiver.client.SpaClientConfiguration;
import com.wingarc.svf.pdfarchiver.client.api.SpaClient;
import com.wingarc.svf.pdfarchiver.client.api.SpaClientException;
import com.wingarc.svf.pdfarchiver.client.api.resource.ArchivesResource;
import com.wingarc.svf.pdfarchiver.client.api.resource.AuthenticationResource;
import com.wingarc.svf.pdfarchiver.webapp.entity.archive.ArchiveParameters;
import com.wingarc.svf.pdfarchiver.webapp.entity.archive.ArchiveResultEntry;

public class C4JApi3Archive {

    /** URLベース */
    private static final String URL_BASE = "http://localhost:44230/spa/service";
    /** ユーザID */
    private static final String USERID = "admin";
    /** パスワード */
    private static final String PASSWORD = "admin";
    /** */
    private static final String DOMAIN = "local";

    /** SpaClient */
    private SpaClient client__ = null;
    /** ログアウト用に保持 */
    private AuthenticationResource auth__ = null;

    /**
     * メインメソッド
     * @param args
     */
    public static void main(String[] args) throws Exception {
        //1.ログイン
        C4JApi3Archive ca = new C4JApi3Archive();
        try {
            ca.login();
            System.out.println("1.ログイン成功");
        } catch (SpaClientException e) {
            System.out.println("ログインに失敗しました。ID/PWを確認してください。");
            e.printStackTrace();
            return;
        }

        //2.アーカイブ
        System.out.println("2.アーカイブ処理 開始");
        ca.archive("C:\\sample.pdf", "/sample");
        System.out.println("2.アーカイブ処理 終了");

        //3.アーカイブ カスタムプロパティを付与
        System.out.println("3.アーカイブ カスタムプロパティ処理 開始");
        //カスタムプロパティをセット
        Map<Integer, String> customProp = new HashMap<>();
        //ID 50 取引金額
        customProp.put(50, "10000");
        //ID 51 取引日付(yyyyMMdd)
        customProp.put(51, "20230413");
        //ID 52 取引先
        customProp.put(52, "ウイング商事株式会社");

        //アーカイブ処理
        ca.archiveCustomProp("C:\\sample.pdf", "/sampleCustomProp", customProp);
        System.out.println("3.アーカイブ カスタムプロパティ処理 終了");

        //9.ログアウト
        ca.logout();
        System.out.println("9.ログアウト 成功");
    }


    /**
     * 引数で指定したファイルを指定のinvoiceAgentのパスに保存します。<br>
     * 指定したパスのフォルダーがない場合は作成します。
     * @param fileFullPath ローカルファイルのフルパス
     * @param acvPath invoiceAgent側の保存フォルダーパス
     * @throws SpaClientException SPA側で発生した例外
     * @throws IOException
     */
    public void archive(String fileFullPath, String acvPath) throws SpaClientException,IOException{

        //ローカルファイル
        File file = new File(fileFullPath);
        String name = file.getName();

        //アーカイブのためのパラメータを作成
        ArchiveParameters parameter = new ArchiveParameters(acvPath);
        //保存時のファイル名
        parameter.setName(name);
        //アーカイブ時にフォルダーが存在しなければ作成する
        parameter.setMkdirs(true);
        //アーカイブ先に同一ファイルが存在した場合は上書きする
        parameter.setOverwrite(true);

        //アーカイブを実行
        ArchivesResource archives = new ArchivesResource(client__);
        ArchiveResultEntry result = archives.add(parameter, file);

        //アーカイブ結果を出力
        //文書ID
        System.out.println("  文書ID : " + result.getId());
        //ファイル名
        System.out.println("  ファイル名 : " + result.getName());
        //プレビューURL
        System.out.println("  プレビューURL : " + result.getUrl());

    }

    /**
     * 引数で指定したファイルを指定のinvoiceAgentのパスにカスタムプロパティを付与して保存します。<br>
     * 指定したパスのフォルダーがない場合は作成します。
     * @param fileFullPath ローカルファイルのフルパス
     * @param acvPath invoiceAgent側の保存フォルダーパス
     * @param customProp カスタムプロパティ情報 keyにカスタムプロパティID、valueに値
     * @throws SpaClientException SPA側で発生した例外
     * @throws IOException
     */
    public void archiveCustomProp(String fileFullPath, String acvPath, Map<Integer, String> customProp) throws SpaClientException,IOException{

        //ローカルファイル
        File file = new File(fileFullPath);
        String name = file.getName();

        //アーカイブのためのパラメータを作成
        ArchiveParameters parameter = new ArchiveParameters(acvPath);
        //保存時のファイル名
        parameter.setName(name);
        //アーカイブ時にフォルダーが存在しなければ作成する
        parameter.setMkdirs(true);
        //アーカイブ先に同一ファイルが存在した場合は上書きする
        parameter.setOverwrite(true);

        //カスタムプロパティ情報
        customProp.forEach((key, value) -> {
            System.out.println("  カスタムプロパティID: " + key + ", 値: " + value);
            parameter.addCustomProperty(key, value);
        });
//        //カスタムプロパティ情報
//        //ID 50 取引金額
//        parameter.addCustomProperty(50, "10000");
//        //ID 51 取引日付
//        parameter.addCustomProperty(51, "20230413");
//        //ID 52 取引先
//        parameter.addCustomProperty(52, "ウイング商事株式会社");

        //アーカイブを実行
        ArchivesResource archives = new ArchivesResource(client__);
        ArchiveResultEntry result = archives.add(parameter, file);

        //アーカイブ結果を出力
        //文書ID
        System.out.println("  文書ID : " + result.getId());
        //ファイル名
        System.out.println("  ファイル名 : " + result.getName());
        //プレビューURL
        System.out.println("  プレビューURL : " + result.getUrl());

    }

    /**
     * ログイン処理
     * @throws Exception
     */
    public void login() throws SpaClientException,IOException {
        try {
            // ログイン
            URL url = new URL(URL_BASE);
            SpaClientConfiguration config = new SpaClientConfiguration();
            client__ = new SpaClient(url, config);

            auth__ = new AuthenticationResource(client__);
            auth__.login(USERID, PASSWORD, DOMAIN);
        } catch (SpaClientException e) {
            throw e;
        } catch(IOException e) {
            throw e;
        }

    }


    /**
     * ログアウト処理
     * @throws SpaClientException
     */
    public void logout() throws SpaClientException, IOException{
        // ログアウト
        auth__.logout();
    }
}
